/**
 * Vars
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var types = {
  HANDLE_CLICK: 'click',
  HANDLE_RESIZE: 'resize',
  HANDLE_SCROLL: 'scroll',
  HANDLE_POPSTATE: 'popstate',
  HANDLE_UNLOAD: 'unload',
  HANDLE_LOAD: 'load',
  HANDLE_BEFORE_UNLOAD: 'beforeunload',
  HANDLE_DOM_CONTENT_LOADED: 'DOMContentLoaded',
  HANDLE_DOM_READY: 'DOMContentLoaded'
};

var docEvents = ['DOMContentLoaded', 'click'];
var typeList = Object.keys(types).reduce(function (memo, type) {
  return memo.concat([type, 'UN' + type]);
}, []);

/**
 * Events
 */

function eventMiddleware(_ref) {
  var _ref$wnd = _ref.wnd;
  var wnd = _ref$wnd === undefined ? window : _ref$wnd;
  var _ref$doc = _ref.doc;
  var doc = _ref$doc === undefined ? document : _ref$doc;

  var map = {};
  var idGen = idGenerator();

  return function (_ref2) {
    var dispatch = _ref2.dispatch;
    var getState = _ref2.getState;
    return function (next) {
      return function (action) {
        return typeList.indexOf(action.type) !== -1 ? Promise.resolve(handle(dispatch, action)) : next(action);
      };
    };
  };

  function handle(dispatch, action) {
    if (action.type.slice(0, 2) !== 'UN') {
      var fn = compose(dispatch, action.payload.cb);

      if (action.type === 'HANDLE_DOM_READY') {
        var hack = doc.documentElement.doScroll;
        if ((hack ? /^loaded|^c/ : /^loaded|^i|^c/).test(doc.readyState)) {
          fn();
          return;
        }
      }

      var evt = types[action.type];
      var el = isDocEvent(evt) ? doc : wnd;
      var id = idGen();

      el.addEventListener(evt, fn);
      map[id] = fn;

      return id;
    } else {
      var evt = types[action.type.slice(0, 2)];
      var el = isDocEvent(evt) ? doc : wnd;
      var id = action.payload.value;

      el.removeEventListener(evt, map[id]);
      delete map[id];
    }
  }
}

function isDocEvent(evt) {
  return docEvents.indexOf(evt) !== -1;
}

function idGenerator() {
  var id = 0;
  return function () {
    return id++;
  };
}

function compose() {
  for (var _len = arguments.length, fns = Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  return fns.reduce(function (memo, fn) {
    return function (arg) {
      return memo(fn(arg));
    };
  }, function (arg) {
    return arg;
  });
}

/**
 * Exports
 */

exports['default'] = eventMiddleware;
module.exports = exports['default'];